# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import argparse

from pypackt import utils
from pypackt.settings import PROJECT_DESC
from pypackt.spider import run_spider


def main():
    parser = argparse.ArgumentParser(description=PROJECT_DESC)
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-c', '--configure',
        help='Configure login and password to www.packtpub.com.',
        action='store_true'
    )
    group.add_argument(
        '-l', '--last',
        help='Show last claimed book.',
        action='store_true'
    )
    group.add_argument(
        '-ls', '--list',
        help='List all books claimed with pypackt.',
        action='store_true'
    )
    group.add_argument(
        '-cr', '--cron',
        help="Add job to user's crontab to claim free ebooks daily.",
        action='store_true'
    )
    group.add_argument(
        '-s', '--show',
        help='Show login settings.',
        action='store_true'
    )

    parsed_args = parser.parse_args()
    choices = {
        parsed_args.configure: utils.set_user_login_data,
        parsed_args.last: utils.show_last,
        parsed_args.list: utils.list_books,
        parsed_args.cron: utils.add_cronjob,
        parsed_args.show: utils.show_user_login_data,
    }
    utils.check_user_login_data()
    choices.get(True, run_spider)()



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
from getpass import getpass
try:
    from ConfigParser import SafeConfigParser
except ImportError:
    from configparser import SafeConfigParser

from six.moves import input

from crontab import CronTab

from pypackt.settings import BOOKS_SECTION
from pypackt.settings import LOGIN_KEY
from pypackt.settings import LOGIN_SECTION
from pypackt.settings import PASS_KEY
from pypackt.settings import USER_CONF_FILE


def get_config_parser():
    """Returns initialized configuration parser.

    :return: parser
    :rtype: SafeConfigParser
    """
    parser = SafeConfigParser()
    parser.read(USER_CONF_FILE)
    return parser


def set_config_option(section, option, value):
    """Sets section's option to a given value and saves configuration to file.

    :param str section: Name of the section to store option under.
    :param str option: Option's name.
    :param value: Option's value.
    """
    parser = get_config_parser()
    parser.set(section, option, value)
    with open(USER_CONF_FILE, 'w') as f:
        parser.write(f)


def get_config_option(section, option):
    """Returns option's value.

    :param str section: Name of the section where option is stored.
    :param str option: Option's name.
    :return: Option's value
    :rtype: str
    """
    parser = get_config_parser()
    return parser.get(section, option)


def set_user_login_data():
    """Sets user's login and password."""
    print('Set Packtpub login details')
    login = input('{}: '.format(LOGIN_KEY.capitalize()))
    password = getpass(str('{}: ').format(PASS_KEY.capitalize()))
    set_config_option(LOGIN_SECTION, LOGIN_KEY, login)
    set_config_option(LOGIN_SECTION, PASS_KEY, password)


def get_user_login_data():
    """Returns user's login and password.

    :rtype: (str, str)
    """
    return (
        get_config_option(LOGIN_SECTION, LOGIN_KEY),
        get_config_option(LOGIN_SECTION, PASS_KEY)
    )


def check_user_login_data():
    """Checks if user's login details have been set."""
    if any(x == 'None' for x in get_user_login_data()):
        print('Login data for packtpub.com is not set - please set it now:')
        set_user_login_data()


def show_user_login_data():
    """Prints user's login and password."""
    print('Packtpub login details')
    print('{0}: {2}\n{1}: {3}'.format(
        LOGIN_KEY.capitalize(), PASS_KEY.capitalize(), *get_user_login_data())
    )


def add_cronjob():
    """Adds pypackt to user's crontab.

    Added job is: @daily pypackt.
    """
    user_cron = CronTab(user=True)
    cmd = 'pypackt'
    if not list(user_cron.find_command(cmd)):
        job = user_cron.new(command=cmd, comment='Job added by pypackt.')
        job.every().dom()
        user_cron.write()
        print('Job "{}" added successfully.'.format(job))
    else:
        print('Job already in crontab.')


def list_books():
    """Prints all books claimed with pypackt."""
    parser = get_config_parser()
    for date, title in parser.items(BOOKS_SECTION):
        print(date, title)


def show_last():
    """Prints title of the most recent book."""
    print(get_config_option(BOOKS_SECTION, 'last'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os

USER_CONF_FILE = os.path.join(
    os.path.abspath(os.path.dirname(__file__)), 'user.ini'
)
LOGIN_SECTION = 'packtpub'
BOOKS_SECTION = 'books'
LOGIN_KEY = 'login'
PASS_KEY = 'password'
PROJECT_URL = 'https://bitbucket.org/kchomski/pypackt'
PROJECT_DESC = (
    'Tool to claim your daily free eBooks at www.packtpub.com with ease.'
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
import os

logging.basicConfig(
    level=logging.ERROR,
    filename=os.path.join(
        os.path.abspath(os.path.dirname(__file__)), 'pypackt.log'
    ),
    filemode='w',
)



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
import logging
from datetime import datetime

import scrapy
from requests.compat import urljoin
from scrapy.crawler import CrawlerProcess

from pypackt.settings import BOOKS_SECTION
from pypackt.settings import PROJECT_URL
from pypackt.utils import get_user_login_data
from pypackt.utils import set_config_option

logger = logging.getLogger(__name__)


class PyPacktSpider(scrapy.Spider):

    name = 'pypackt'
    base_url = 'https://www.packtpub.com'
    start_urls = [urljoin(base_url, 'packt/offers/free-learning')]
    book = {}

    def start_requests(self):
        """Logs in into user's account before claiming ebook."""
        login, password = get_user_login_data()
        form_data = {
            'email': login,
            'password': password,
            'op': 'Login',
            'form_id': 'packt_user_login_form',
        }
        yield scrapy.FormRequest(
            url=self.start_urls[0],
            formdata=form_data,
            callback=self.parse
        )

    def parse(self, response):
        """Parses response to extract ebook's title and URL."""
        self.book = {
            'title':
                response.css('.dotd-title h2::text').extract_first().strip(),
            'url': urljoin(
                self.base_url,
                response.css('.twelve-days-claim::attr(href)').extract_first()),
        }
        yield scrapy.Request(
            url=self.book['url'],
            callback=self.on_success,
            errback=self.on_fail
        )

    def on_success(self, response):
        logger.info(self.book['title'])
        print(self.book['title'])
        set_config_option(BOOKS_SECTION, 'last', self.book['title'])
        today = datetime.now().date().isoformat()
        set_config_option(BOOKS_SECTION, today, self.book['title'])

    def on_fail(self, response):
        print(
            'Something went wrong - please check if you can login manually '
            'at {}\nIf logging in manually works, '
            'please raise an issue at: {}'.format(self.base_url, PROJECT_URL)
        )


def run_spider():
    process = CrawlerProcess({
        'USER_AGENT': 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)',
        'LOG_ENABLED': False,
    })
    process.crawl(PyPacktSpider)
    process.start()



# -*- coding: utf-8 -*-
import os
from setuptools import find_packages
from setuptools import setup

try:
    from pypandoc import convert
    def read_md(f):
        return convert(f, 'rst')
except ImportError:
    def read_md(f):
        return open(f, 'r').read()


from pypackt.settings import PROJECT_DESC
from pypackt.settings import PROJECT_URL

README_PATH = os.path.join(
    os.path.abspath(os.path.dirname(__file__)), 'README.md'
)


setup(
    name='pypackt',
    version='1.0.0',
    description=PROJECT_DESC,
    long_description=read_md(README_PATH),
    url=PROJECT_URL,
    author='Krzysztof Chomski',
    author_email='krzysztof.chomski@gmail.com',
    license='MIT',
    keywords=['ebooks', 'learning', 'education'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Education',
    ],
    packages=find_packages(),
    install_requires=['python-crontab', 'requests', 'scrapy'],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'pypackt=pypackt.run:main',
        ],
    },
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-mock']
)
